require "ffxiv_scraper/version"
require 'ffxiv_scraper/lodestone/common'
require 'open-uri'
require 'nokogiri'

module FFXIVScraper
  module Lodestone
    class News
      include Common
      CATEGORY = {notice: '1', maintenance: '2', update:'3', status: '4'}.map(&:freeze).to_h.freeze

      def initialize(category = :notice)
        @category = category
        set_locale
        unless CATEGORY.key?(category)
          raise "not found category. [" + category.to_s + "]"
        end
      end

      def get_news_url
        return get_lodestone_url + "/news/category/#{CATEGORY[@category]}"
      end

      def news_list(backnumber = 0)
        news_list = []
        news_url = backnumber > 0 ? get_news_url << "?page=#{backnumber}" : get_news_url
        
        doc = open_html(news_url)
        doc.xpath('//li[@class="news__list"]').each do |news|
          news_list << NewsContent.new(@lang, news.xpath('.//time').text[/strftime\((\d+),/,1].to_i, 
                                        news.xpath('.//p').text, 
                                        news.xpath('.//a/@href').text, 
                                        news.xpath('.//p/span').text)
        end
        news_list.sort { |a, b| b.created_at <=> a.created_at }
      end

      private
      module Content
        def get_detail(detail_path)
          doc = open_html(get_base_url + detail_path)
          
          detail = doc.xpath('//article')
          relationships = []
          detail.xpath('.//li[@class="news__list"]').each do |relation|
            relationship = NewsContent.new(@lang, relation.xpath('.//time').text[/strftime\((\d+),/,1].to_i,
                                            relation.xpath('.//p/span').text + relation.xpath('.//p/text()[2]').text.strip, 
                                            relation.xpath('./a/@href').text, 
                                            relation.xpath('.//p/span').text)
            relationships << relationship
          end
          return DetailContent.new(@lang, detail.xpath('./div[@class="news__detail__wrapper"]').text, 
                                    detail.xpath('.//li[@class="news__nav__next"]/a/@href').text, 
                                    detail.xpath('.//li[@class="news__nav__prev"]/a/@href').text,
                                    relationships)
        end
      end

      DetailContent = Struct.new("DetailContent", :text, :next_path, :prev_path, :relationships) do 
        include Common
        include Content

        def initialize(lang, text, next_path, prev_path, relationships)
          set_locale(lang)
          super(text, next_path, prev_path, relationships)
        end
        
        def detail_next
          get_detail(next_path)
        end

        def detail_prev
          get_detail(prev_path)
        end
      end

      NewsContent = Struct.new("NewsContent", :created_at, :title, :detail_path, :tag) do
        include Common
        include Content

        def initialize(lang, created_at, title, detail_path, tag)
          set_locale(lang)
          super(created_at, title, detail_path, tag)
        end

        def detail
          get_detail(detail_path)
        end
      end
    end
  end
end