require "ffxiv_scraper/version"
require 'open-uri'
require 'nokogiri'

module FFXIVScraper
  module Lodestone
    class WorldStatus
      include Common
      INTERVAL = 10.freeze

      def initialize
        set_locale
        @needs_update = true
        @update_at = Time.now
        @statuses = []
      end

      def set_locale(lang = :ja)
        @lang = lang
        @needs_update = true
      end

      def get_statuses
        if (Time.now - @update_at) > INTERVAL
          @needs_update = true
        end

        if @needs_update
          @statuses = []
          doc = open_html(get_lodestone_url << "/worldstatus/")
          dc_name = ""
          doc.xpath('//h2[@class="heading--md parts__space--reset"]|//div[@class="item-list__worldstatus"]').each do |content|
            world = ""
            status = ""
            if content.name == "h2"
              dc_name = content.text.downcase
            else
              @statuses << Status.new(dc_name, content.xpath("./h3").text.downcase, content.xpath("./p").text.strip.downcase)
            end
          end
          @update_at = Time.now
          @needs_update = false
        end
        return @statuses
      end

      def get_status(world_name)
        get_statuses
        @statuses.find { |a| a.world == world_name.downcase }
      end

      def get_dc_statuses(dc_name)
        get_statuses
        @statuses.select { |a| a.dc == dc_name.downcase }
      end

      private
      Status = Struct.new("WorldStatus", :dc, :world, :status)
    end
  end
end
